/* $Id: cfunc.tpl,v 1.1 91/03/18 19:01:04 bill Exp $ */
/*.......1.........2.........3.........4.........5.........6.........7.........8
================================================================================

FILE s_xfer/cpe.mod

Copyright 2023
Arpad Bürmen
All Rights Reserved


AUTHORS                      

    30 March 2023    Arpad Bürmen


SUMMARY

    This file contains the implementation of the CPE element model 
    for AC analysis. 


===============================================================================*/

#include <math.h>

#define PI 3.14159265358979323846

void cm_cpe(ARGS)  /* structure holding parms, inputs, outputs, etc.     */
{
    int i;                       /* generic loop counter index */
	int den_size;                /* size of the denominator coefficient array */
	int num_size;                /* size of the numerator coefficient array */ 

    char *num_size_error="\n***ERROR***\nS_XFER: Numerator coefficient array size greater than\ndenominator coefficiant array size.\n";
    
    double q, n;
    double vcpe, icpe;
    
    Mif_Complex_t ac_gain;


    /** Retrieve frequently used parameters (used by all analyses)... **/

    q = PARAM(q);
    n = PARAM(n);                    
    
    vcpe=INPUT(cpe);
    icpe=0;
    
    /* if ( num_size > den_size ) {
        cm_message_send(num_size_error);
        return;
    } */

    /** Test for INIT; if so, allocate storage, otherwise, retrieve previous       **/
    /** timepoint input values as necessary in subsequent analysis sections...     **/

    if (INIT==1) {  /* First pass...allocate storage */
        
    }else { /* Allocation was not necessary...retrieve previous values */
    
    }


    /** Test for TIME=0.0; if so, initialize...       **/

    if (TIME == 0.0) {  /* First pass...set initial conditions... */
        
    }
                                 

    /**** DC & Transient Analyses **************************/
    if (ANALYSIS != MIF_AC) {     
        
        /* Treat as resistor */
        OUTPUT(cpe) = icpe;
        PARTIAL(cpe,cpe) = 0;

    }

    /**** AC Analysis ************************************/
    else {                    
	
	double omega_n;
	double jn_angle;
	
	/* Compute omega tau raised to power of n */
	omega_n=pow(RAD_FREQ, n);
	
	/* Compute j raised to power of n */
	jn_angle=(PI/2)*n;
	
	/* Convert to gain real, imag */
	ac_gain.real = q * omega_n * cos(jn_angle);
	ac_gain.imag = q * omega_n * sin(jn_angle);
	
        AC_GAIN(cpe,cpe) = ac_gain;
    }
}







