#ifndef __HASH_DEFINED
#define __HASH_DEFINED

typedef void StringPtrHash;

// Create a hash with string for key and pointer for value
// Returns a hash handle
StringPtrHash *spHashCreate();

// Destroy a hash
void spHashDestroy(StringPtrHash *h);

// Find element by key, return it in value. 
// Return 0 if notwhing was found, otherwise return 1. 
int spHashLookup(StringPtrHash *h, const char *key, void *value);

// Insert an element, return previous element at the given key in oldValue. 
// Return 0 if nothing was in the hash under the given key, 
// otherwise return 1, 
int spHashInsert(StringPtrHash *h, const char *k, void *value, void *oldValue);

// Remove an element. 
// Return 1 if there was an element to remove, otherwise return 0. 
int spHashRemove(StringPtrHash *h, const char *k);

// Apply void func(void *) to all values
void spHashIterate(StringPtrHash *h, void (*func)(void *));

// Remove all entries
void spHashClear(StringPtrHash *h);


// Hash with string for key and integer for value
typedef void StringIntHash;
StringIntHash *siHashCreate();
void siHashDestroy(StringIntHash *h);
int siHashLookup(StringIntHash *h, const char *key, int *value);
int siHashInsert(StringIntHash *h, const char *k, int value, int *oldValue);
int siHashRemove(StringIntHash *h, const char *k);
void siHashIterate(StringIntHash *h, void (*func)(int));
void siHashClear(StringIntHash *h);


// Hash with integer for key and pointer for value
typedef void * IntPtrHash;
IntPtrHash *ipHashCreate();
void ipHashDestroy(IntPtrHash *h);
int ipHashLookup(IntPtrHash *h, const int key, void *value);
int ipHashInsert(IntPtrHash *h, const int k, void *value, void *oldValue);
int ipHashRemove(IntPtrHash *h, const int k);
void ipHashIterate(IntPtrHash *h, void (*func)(void *));
void ipHashClear(IntPtrHash *h);


// Hash with pointer for key and integer for value
typedef void PtrIntHash;
PtrIntHash *piHashCreate();
void piHashDestroy(PtrIntHash *h);
int piHashLookup(PtrIntHash *h, void * const key, int *value);
int piHashInsert(PtrIntHash *h, void * const k, int value, int *oldValue);
int piHashRemove(PtrIntHash *h, void * const k);
void piHashIterate(PtrIntHash *h, void (*func)(int));
void piHashClear(PtrIntHash *h);

#endif
